@@ -1,5 +1,5 @@
 #!/usr/bin/python2
-"""Preferences Editor for Terminator.
+"""Preferences Editor for Terminator. 
 
 Load a UIBuilder config file, display it,
 populate it with our current config, then optionally read that back out and
@@ -8,8 +8,7 @@ write it to a config file
 """
 
 import os
-import gtk
-import gobject
+from gi.repository import GObject, Gtk, Gdk
 
 from util import dbg, err
 import config
@@ -160,7 +159,7 @@ class PrefsEditor:
         self.config = config.Config()
         self.config.base.reload()
         self.term = term
-        self.builder = gtk.Builder()
+        self.builder = Gtk.Builder()
         self.builder.set_translation_domain(APP_NAME)
         self.keybindings = Keybindings()
         try:
@@ -177,12 +176,12 @@ class PrefsEditor:
         self.builder.add_from_string(gladedata)
         self.window = self.builder.get_object('prefswin')
 
-        icon_theme = gtk.icon_theme_get_default()
+        icon_theme = Gtk.IconTheme.get_default()
         if icon_theme.lookup_icon('terminator-preferences', 48, 0):
             self.window.set_icon_name('terminator-preferences')
         else:
             dbg('Unable to load Terminator preferences icon')
-            icon = self.window.render_icon(gtk.STOCK_DIALOG_INFO, gtk.ICON_SIZE_BUTTON)
+            icon = self.window.render_icon(Gtk.STOCK_DIALOG_INFO, Gtk.IconSize.BUTTON)
             self.window.set_icon(icon)
 
         self.layouteditor = LayoutEditor(self.builder)
@@ -355,7 +354,7 @@ class PrefsEditor:
         ## Keybindings tab
         widget = guiget('keybindingtreeview')
         liststore = widget.get_model()
-        liststore.set_sort_column_id(0, gtk.SORT_ASCENDING)
+        liststore.set_sort_column_id(0, Gtk.SortType.ASCENDING)
         keybindings = self.config['keybindings']
         for keybinding in keybindings:
             keyval = 0
@@ -412,9 +411,6 @@ class PrefsEditor:
         # Allow bold text
         widget = guiget('allow_bold_checkbutton')
         widget.set_active(self.config['allow_bold'])
-        # Anti-alias
-        widget = guiget('antialias_checkbutton')
-        widget.set_active(self.config['antialias'])
         # Icon terminal bell
         widget = guiget('icon_bell_checkbutton')
         widget.set_active(self.config['icon_bell'])
@@ -433,9 +429,15 @@ class PrefsEditor:
         # Copy on selection
         widget = guiget('copy_on_selection')
         widget.set_active(self.config['copy_on_selection'])
+        # Rewrap on resize
+        widget = guiget('rewrap_on_resize_checkbutton')
+        widget.set_active(self.config['rewrap_on_resize'])
         # Word chars
         widget = guiget('word_chars_entry')
         widget.set_text(self.config['word_chars'])
+        # Word char support was missing from vte 0.38, hide from the UI
+        if not hasattr(self.term.vte, 'set_word_char_exceptions'):
+            guiget('word_chars_hbox').hide()
         # Cursor shape
         widget = guiget('cursor_shape_combobox')
         if self.config['cursor_shape'] == 'underline':
@@ -458,22 +460,19 @@ class PrefsEditor:
         widget = guiget('cursor_color')
         widget.set_sensitive(not self.config['cursor_color_fg'])
         try:
-             widget.set_color(gtk.gdk.Color(self.config['cursor_color']))
+            widget.set_color(Gdk.color_parse(self.config['cursor_color']))
         except (ValueError, TypeError):
             try:
                 self.config['cursor_color'] = self.config['foreground_color']
-                widget.set_color(gtk.gdk.Color(self.config['cursor_color']))
+                widget.set_color(Gdk.color_parse(self.config['cursor_color']))
             except ValueError:
                 self.config['cursor_color'] = "#FFFFFF"
-                widget.set_color(gtk.gdk.Color(self.config['cursor_color']))
+                widget.set_color(Gdk.color_parse(self.config['cursor_color']))
 
         ## Command tab
         # Login shell
         widget = guiget('login_shell_checkbutton')
         widget.set_active(self.config['login_shell'])
-        # Login records
-        widget = guiget('update_records_checkbutton')
-        widget.set_active(self.config['update_records'])
         # Use Custom command
         widget = guiget('use_custom_command_checkbutton')
         widget.set_active(self.config['use_custom_command'])
@@ -514,14 +513,14 @@ class PrefsEditor:
         # NOTE: The scheme is set in the GUI widget after the fore/back colours
         # Foreground color
         widget = guiget('foreground_colorpicker')
-        widget.set_color(gtk.gdk.Color(self.config['foreground_color']))
+        widget.set_color(Gdk.color_parse(self.config['foreground_color']))
         if scheme == 'custom':
             widget.set_sensitive(True)
         else:
             widget.set_sensitive(False)
         # Background color
         widget = guiget('background_colorpicker')
-        widget.set_color(gtk.gdk.Color(self.config['background_color']))
+        widget.set_color(Gdk.color_parse(self.config['background_color']))
         if scheme == 'custom':
             widget.set_sensitive(True)
         else:
@@ -545,7 +544,7 @@ class PrefsEditor:
         colourpalette = self.config['palette'].split(':')
         for i in xrange(1, 17):
             widget = guiget('palette_colorpicker_%d' % i)
-            widget.set_color(gtk.gdk.Color(colourpalette[i - 1]))
+            widget.set_color(Gdk.color_parse(colourpalette[i - 1]))
         # Now set the palette selector widget
         widget = guiget('palette_combobox')
         widget.set_active(self.palettevalues[palette])
@@ -554,7 +553,7 @@ class PrefsEditor:
             'title_receive_fg_color', 'title_receive_bg_color',
             'title_inactive_fg_color', 'title_inactive_bg_color']:
             widget = guiget(bit)
-            widget.set_color(gtk.gdk.Color(self.config[bit]))
+            widget.set_color(Gdk.color_parse(self.config[bit]))
         # Inactive terminal shading
         widget = guiget('inactive_color_offset')
         widget.set_value(float(self.config['inactive_color_offset']))
@@ -570,20 +569,9 @@ class PrefsEditor:
         # Radio values
         if self.config['background_type'] == 'solid':
             guiget('solid_radiobutton').set_active(True)
-        elif self.config['background_type'] == 'image':
-            guiget('image_radiobutton').set_active(True)
         elif self.config['background_type'] == 'transparent':
             guiget('transparent_radiobutton').set_active(True)
         self.update_background_tab()
-        # Background image file
-        if self.config['background_image'] != '':
-            widget = guiget('background_image_filechooser')
-            if self.config['background_image'] is not None and \
-               self.config['background_image'] != '':
-                widget.set_filename(self.config['background_image'])
-        # Background image scrolls
-        widget = guiget('scroll_background_checkbutton')
-        widget.set_active(self.config['scroll_background'])
         # Background shading
         widget = guiget('background_darkness_scale')
         widget.set_value(float(self.config['background_darkness']))
@@ -610,9 +598,6 @@ class PrefsEditor:
         # Scroll on keystroke
         widget = guiget('scroll_on_keystroke_checkbutton')
         widget.set_active(self.config['scroll_on_keystroke'])
-        # Scroll in alternate mode
-        widget = guiget('alternate_screen_scroll_checkbutton')
-        widget.set_active(self.config['alternate_screen_scroll'])
 
         ## Compatibility tab
         # Backspace key
@@ -726,11 +711,6 @@ class PrefsEditor:
         self.config['allow_bold'] = widget.get_active()
         self.config.save()
 
-    def on_antialias_checkbutton_toggled(self, widget):
-        """Anti-alias setting changed"""
-        self.config['antialias'] = widget.get_active()
-        self.config.save()
-
     def on_show_titlebar_toggled(self, widget):
         """Show titlebar setting changed"""
         self.config['show_titlebar'] = widget.get_active()
@@ -741,6 +721,11 @@ class PrefsEditor:
         self.config['copy_on_selection'] = widget.get_active()
         self.config.save()
 
+    def on_rewrap_on_resize_toggled(self, widget):
+        """Rewrap on resize setting changed"""
+        self.config['rewrap_on_resize'] = widget.get_active()
+        self.config.save()
+
     def on_putty_paste_style_toggled(self, widget):
         """Putty paste style setting changed"""
         self.config['putty_paste_style'] = widget.get_active()
@@ -781,21 +766,11 @@ class PrefsEditor:
         self.config['login_shell'] = widget.get_active()
         self.config.save()
 
-    def on_update_records_checkbutton_toggled(self, widget):
-        """Update records setting changed"""
-        self.config['update_records'] = widget.get_active()
-        self.config.save()
-
     def on_scroll_background_checkbutton_toggled(self, widget):
         """Scroll background setting changed"""
         self.config['scroll_background'] = widget.get_active()
         self.config.save()
 
-    def on_alternate_screen_scroll_checkbutton_toggled(self, widget):
-        """Scroll in alt-mode setting changed"""
-        self.config['alternate_screen_scroll'] = widget.get_active()
-        self.config.save()
-
     def on_scroll_on_keystroke_checkbutton_toggled(self, widget):
         """Scroll on keystrong setting changed"""
         self.config['scroll_on_keystroke'] = widget.get_active()
@@ -880,11 +855,6 @@ class PrefsEditor:
         self.config['background_darkness'] = value
         self.config.save()
 
-    def on_background_image_filechooser_file_set(self, widget):
-        """Background image setting changed"""
-        self.config['background_image'] = widget.get_filename()
-        self.config.save()
-
     def on_palette_combobox_changed(self, widget):
         """Palette selector changed"""
         value = None
@@ -910,7 +880,7 @@ class PrefsEditor:
             for num in xrange(1, 17):
                 # Update the visible elements
                 picker = guiget('palette_colorpicker_%d' % num)
-                picker.set_color(gtk.gdk.Color(palettebits[num - 1]))
+                picker.set_color(Gdk.color_parse(palettebits[num - 1]))
         elif value == 'custom':
             palettebits = []
             for num in xrange(1, 17):
@@ -982,14 +952,14 @@ class PrefsEditor:
         self.config['cursor_color_fg'] = not customwidget.get_active()
         
         try:
-            colorwidget.set_color(gtk.gdk.color_parse(self.config['cursor_color']))
+            colorwidget.set_color(Gdk.color_parse(self.config['cursor_color']))
         except (ValueError, TypeError):
             try:
                 self.config['cursor_color'] = self.config['foreground_color']
-                colorwidget.set_color(gtk.gdk.color_parse(self.config['cursor_color']))
+                colorwidget.set_color(Gdk.color_parse(self.config['cursor_color']))
             except ValueError:
                 self.config['cursor_color'] = "#FFFFFF"
-                colorwidget.set_color(gtk.gdk.color_parse(self.config['cursor_color']))
+                colorwidget.set_color(Gdk.color_parse(self.config['cursor_color']))
         self.config.save()
 
     def on_cursor_color_color_set(self, widget):
@@ -1206,7 +1176,7 @@ class PrefsEditor:
         name = model.get_value(rowiter, 0)
 
         if self.config.replace_layout(name, current_layout):
-            treeview.set_cursor(model.get_path(rowiter), focus_column=treeview.get_column(0), start_editing=False)
+            treeview.set_cursor(model.get_path(rowiter), column=treeview.get_column(0), start_editing=False)
         self.config.save()
 
     def on_layoutremovebutton_clicked(self, _button):
@@ -1310,19 +1280,11 @@ class PrefsEditor:
         transwidget = guiget('transparent_radiobutton')
         if transwidget.get_active() == True:
             backtype = 'transparent'
-        elif imagewidget.get_active() == True:
-            backtype = 'image'
         else:
             backtype = 'solid'
         self.config['background_type'] = backtype
         self.config.save()
 
-        if backtype == 'image':
-            guiget('background_image_filechooser').set_sensitive(True)
-            guiget('scroll_background_checkbutton').set_sensitive(True)
-        else:
-            guiget('background_image_filechooser').set_sensitive(False)
-            guiget('scroll_background_checkbutton').set_sensitive(False)
         if backtype in ('transparent', 'image'):
             guiget('darken_background_scale').set_sensitive(True)
         else:
@@ -1391,18 +1353,7 @@ class PrefsEditor:
 
     def on_profile_name_edited(self, cell, path, newtext):
         """Update a profile name"""
-        oldname_broken = cell.get_property('text')
-        
-        guiget = self.builder.get_object
-        treeview = guiget('profilelist')
-        treeselection = treeview.get_selection()
-        treeselection.select_path(path)
-        (model, pathlist) = treeselection.get_selected_rows()
-        tree_iter = model.get_iter(pathlist[0])
-        oldname = model.get_value(tree_iter,0)
-        if oldname != oldname_broken:
-            dbg('edited signal provides the wrong cell: %s != %s' %(oldname, oldname_broken))
-
+        oldname = cell.get_property('text')
         if oldname == newtext or oldname == 'default':
             return
         dbg('PrefsEditor::on_profile_name_edited: Changing %s to %s' %
@@ -1440,18 +1391,7 @@ class PrefsEditor:
 
     def on_layout_name_edited(self, cell, path, newtext):
         """Update a layout name"""
-        oldname_broken = cell.get_property('text')
-
-        guiget = self.builder.get_object
-        treeview = guiget('layoutlist')
-        treeselection = treeview.get_selection()
-        treeselection.select_path(path)
-        (model, pathlist) = treeselection.get_selected_rows()
-        tree_iter = model.get_iter(pathlist[0])
-        oldname = model.get_value(tree_iter,0)
-        if oldname != oldname_broken:
-            dbg('edited signal provides the wrong cell: %s != %s' %(oldname, oldname_broken))
-
+        oldname = cell.get_property('text')
         if oldname == newtext or oldname == 'default':
             return
         dbg('Changing %s to %s' % (oldname, newtext))
@@ -1500,8 +1440,8 @@ class PrefsEditor:
             err('Unknown colourscheme value: %s' % value)
             return
 
-        fore.set_color(gtk.gdk.Color(forecol))
-        back.set_color(gtk.gdk.Color(backcol))
+        fore.set_color(Gdk.color_parse(forecol))
+        back.set_color(Gdk.color_parse(backcol))
 
         self.config['foreground_color'] = forecol
         self.config['background_color'] = backcol
@@ -1532,7 +1472,7 @@ class PrefsEditor:
         liststore.set(celliter, 2, key, 3, mods)
 
         binding = liststore.get_value(liststore.get_iter(path), 0)
-        accel = gtk.accelerator_name(key, mods)
+        accel = Gtk.accelerator_name(key, mods)
         self.config['keybindings'][binding] = accel
         self.config.save()
 
@@ -1724,4 +1664,4 @@ if __name__ == '__main__':
     TERM = terminal.Terminal()
     PREFEDIT = PrefsEditor(TERM)
 
-    gtk.main()
+    Gtk.main()
