@@ -4,7 +4,7 @@
 """terminal_popup_menu.py - classes necessary to provide a terminal context 
 menu"""
 
-import gtk
+from gi.repository import Gtk
 
 from version import APP_NAME
 from translation import _
@@ -31,7 +31,8 @@ class TerminalPopupMenu(object):
         """Display the context menu"""
         terminal = self.terminal
 
-        menu = gtk.Menu()
+        menu = Gtk.Menu()
+        self.popup_menu = menu
         url = None
         button = None
         time = None
@@ -39,14 +40,14 @@ class TerminalPopupMenu(object):
         self.config.set_profile(terminal.get_profile())
 
         if event:
-            url = terminal.check_for_url(event)
+            url = terminal.vte.match_check_event(event)
             button = event.button
             time = event.time
         else:
             time = 0
             button = 3
 
-        if url:
+        if url and url[0]:
             dbg("URL matches id: %d" % url[1])
             if not url[1] in terminal.matches.values():
                 err("Unknown URL match id: %d" % url[1])
@@ -84,115 +85,115 @@ class TerminalPopupMenu(object):
             if not namecopy:
                 namecopy = _('_Copy address')
 
-            icon = gtk.image_new_from_stock(gtk.STOCK_JUMP_TO,
-                                            gtk.ICON_SIZE_MENU)
-            item = gtk.ImageMenuItem(nameopen)
+            icon = Gtk.Image.new_from_stock(Gtk.STOCK_JUMP_TO,
+                                            Gtk.IconSize.MENU)
+            item = Gtk.ImageMenuItem.new_with_mnemonic(nameopen)
             item.set_property('image', icon)
             item.connect('activate', lambda x: terminal.open_url(url, True))
             menu.append(item)
 
-            item = gtk.MenuItem(namecopy)
+            item = Gtk.MenuItem.new_with_mnemonic(namecopy)
             item.connect('activate', 
-                         lambda x: terminal.clipboard.set_text(terminal.prepare_url(url)))
+                         lambda x: terminal.clipboard.set_text(terminal.prepare_url(url), len(terminal.prepare_url(url))))
             menu.append(item)
 
-            menu.append(gtk.MenuItem())
+            menu.append(Gtk.MenuItem())
 
-        item = gtk.ImageMenuItem(gtk.STOCK_COPY)
+        item = Gtk.ImageMenuItem.new_with_mnemonic(_('_Copy'))
         item.connect('activate', lambda x: terminal.vte.copy_clipboard())
         item.set_sensitive(terminal.vte.get_has_selection())
         menu.append(item)
 
-        item = gtk.ImageMenuItem(gtk.STOCK_PASTE)
+        item = Gtk.ImageMenuItem.new_with_mnemonic(_('_Paste'))
         item.connect('activate', lambda x: terminal.paste_clipboard())
         menu.append(item)
 
-        menu.append(gtk.MenuItem())
+        menu.append(Gtk.MenuItem())
 
         if not terminal.is_zoomed():
-            item = gtk.ImageMenuItem(_('Split H_orizontally'))
-            image = gtk.Image()
-            image.set_from_icon_name(APP_NAME + '_horiz', gtk.ICON_SIZE_MENU)
+            item = Gtk.ImageMenuItem.new_with_mnemonic(_('Split H_orizontally'))
+            image = Gtk.Image()
+            image.set_from_icon_name(APP_NAME + '_horiz', Gtk.IconSize.MENU)
             item.set_image(image)
             if hasattr(item, 'set_always_show_image'):
                 item.set_always_show_image(True)
             item.connect('activate', lambda x: terminal.emit('split-horiz',
-                self.terminator.pid_cwd(self.terminal.pid)))
+                self.terminal.get_cwd()))
             menu.append(item)
 
-            item = gtk.ImageMenuItem(_('Split V_ertically'))
-            image = gtk.Image()
-            image.set_from_icon_name(APP_NAME + '_vert', gtk.ICON_SIZE_MENU)
+            item = Gtk.ImageMenuItem.new_with_mnemonic(_('Split V_ertically'))
+            image = Gtk.Image()
+            image.set_from_icon_name(APP_NAME + '_vert', Gtk.IconSize.MENU)
             item.set_image(image)
             if hasattr(item, 'set_always_show_image'):
                 item.set_always_show_image(True)
             item.connect('activate', lambda x: terminal.emit('split-vert',
-                self.terminator.pid_cwd(self.terminal.pid)))
+                self.terminal.get_cwd()))
             menu.append(item)
 
-            item = gtk.MenuItem(_('Open _Tab'))
+            item = Gtk.MenuItem.new_with_mnemonic(_('Open _Tab'))
             item.connect('activate', lambda x: terminal.emit('tab-new', False,
                 terminal))
             menu.append(item)
 
             if self.terminator.debug_address is not None:
-                item = gtk.MenuItem(_('Open _Debug Tab'))
+                item = Gtk.MenuItem.new_with_mnemonic(_('Open _Debug Tab'))
                 item.connect('activate', lambda x:
                         terminal.emit('tab-new', True, terminal))
                 menu.append(item)
 
-            menu.append(gtk.MenuItem())
+            menu.append(Gtk.MenuItem())
 
-        item = gtk.ImageMenuItem(gtk.STOCK_CLOSE)
+        item = Gtk.ImageMenuItem.new_with_mnemonic(_('_Close'))
         item.connect('activate', lambda x: terminal.close())
         menu.append(item)
 
-        menu.append(gtk.MenuItem())
+        menu.append(Gtk.MenuItem())
 
         if not terminal.is_zoomed():
             sensitive = not terminal.get_toplevel() == terminal.get_parent()
 
-            item = gtk.MenuItem(_('_Zoom terminal'))
+            item = Gtk.MenuItem.new_with_mnemonic(_('_Zoom terminal'))
             item.connect('activate', terminal.zoom)
             item.set_sensitive(sensitive)
             menu.append(item)
 
-            item = gtk.MenuItem(_('Ma_ximize terminal'))
+            item = Gtk.MenuItem.new_with_mnemonic(_('Ma_ximize terminal'))
             item.connect('activate', terminal.maximise)
             item.set_sensitive(sensitive)
             menu.append(item)
 
-            menu.append(gtk.MenuItem())
+            menu.append(Gtk.MenuItem())
         else:
-            item = gtk.MenuItem(_('_Restore all terminals'))
+            item = Gtk.MenuItem.new_with_mnemonic(_('_Restore all terminals'))
             item.connect('activate', terminal.unzoom)
             menu.append(item)
 
-            menu.append(gtk.MenuItem())
+            menu.append(Gtk.MenuItem())
 
         if self.config['show_titlebar'] == False:
-            item = gtk.MenuItem(_('Grouping'))
+            item = Gtk.MenuItem.new_with_mnemonic(_('Grouping'))
             submenu = self.terminal.populate_group_menu()
             submenu.show_all()
             item.set_submenu(submenu)
             menu.append(item)
-            menu.append(gtk.MenuItem())
+            menu.append(Gtk.MenuItem())
 
-        item = gtk.CheckMenuItem(_('Show _scrollbar'))
+        item = Gtk.CheckMenuItem.new_with_mnemonic(_('Show _scrollbar'))
         item.set_active(terminal.scrollbar.get_property('visible'))
         item.connect('toggled', lambda x: terminal.do_scrollbar_toggle())
         menu.append(item)
 
-        if hasattr(gtk, 'Builder'):
-            item = gtk.MenuItem(_('_Preferences'))
+        if hasattr(Gtk, 'Builder'):  # VERIFY FOR GTK3: is this ever false?
+            item = Gtk.MenuItem.new_with_mnemonic(_('_Preferences'))
             item.connect('activate', lambda x: PrefsEditor(self.terminal))
             menu.append(item)
 
         profilelist = self.config.list_profiles()
 
         if len(profilelist) > 1:
-            item = gtk.MenuItem(_('Profiles'))
-            submenu = gtk.Menu()
+            item = Gtk.MenuItem.new_with_mnemonic(_('Profiles'))
+            submenu = Gtk.Menu()
             item.set_submenu(submenu)
             menu.append(item)
 
@@ -201,7 +202,7 @@ class TerminalPopupMenu(object):
             group = None
 
             for profile in profilelist:
-                item = gtk.RadioMenuItem(group, profile.capitalize())
+                item = Gtk.RadioMenuItem(profile.capitalize(), group)
                 if profile == current:
                     item.set_active(True)
                 item.connect('activate', terminal.force_set_profile, profile)
@@ -218,7 +219,7 @@ class TerminalPopupMenu(object):
                 menuplugin.callback(menuitems, menu, terminal)
             
             if len(menuitems) > 0:
-                menu.append(gtk.MenuItem())
+                menu.append(Gtk.MenuItem())
 
             for menuitem in menuitems:
                 menu.append(menuitem)
@@ -226,7 +227,7 @@ class TerminalPopupMenu(object):
             err('TerminalPopupMenu::show: %s' % ex)
 
         menu.show_all()
-        menu.popup(None, None, None, button, time)
+        menu.popup(None, None, None, None, button, time)
 
         return(True)
 
@@ -235,9 +236,9 @@ class TerminalPopupMenu(object):
         """Add the encoding list to the menu"""
         terminal = self.terminal
         active_encodings = terminal.config['active_encodings']
-        item = gtk.MenuItem (_("Encodings"))
+        item = Gtk.MenuItem.new_with_mnemonic(_("Encodings"))
         menu.append (item)
-        submenu = gtk.Menu ()
+        submenu = Gtk.Menu ()
         item.set_submenu (submenu)
         encodings = TerminatorEncoding ().get_list ()
         encodings.sort (lambda x, y: cmp (x[2].lower (), y[2].lower ()))
@@ -257,7 +258,7 @@ class TerminalPopupMenu(object):
             else:
                 extratext = ""
     
-            radioitem = gtk.RadioMenuItem (group, _(encoding) + extratext)
+            radioitem = Gtk.RadioMenuItem (_(encoding) + extratext, group)
     
             if encoding == current_encoding:
                 radioitem.set_active (True)
@@ -269,11 +270,11 @@ class TerminalPopupMenu(object):
                                encoding)
             submenu.append (radioitem)
     
-        item = gtk.MenuItem (_("Other Encodings"))
+        item = Gtk.MenuItem.new_with_mnemonic(_("Other Encodings"))
         submenu.append (item)
         #second level
     
-        submenu = gtk.Menu ()
+        submenu = Gtk.Menu ()
         item.set_submenu (submenu)
         group = None
     
@@ -286,7 +287,7 @@ class TerminalPopupMenu(object):
             else:
                 label = "%s %s" % (encoding[2], encoding[1])
     
-            radioitem = gtk.RadioMenuItem (group, label)
+            radioitem = Gtk.RadioMenuItem (label, group)
             if group is None:
                 group = radioitem
     
