@@ -17,7 +17,8 @@
 """Terminator.util - misc utility functions"""
 
 import sys
-import gtk
+from gi.repository import Gtk, Gdk
+import cairo
 import os
 import pwd
 import inspect
@@ -71,8 +72,8 @@ def gerr(message = None):
     """Display a graphical error. This should only be used for serious
     errors as it will halt execution"""
 
-    dialog = gtk.MessageDialog(None, gtk.DIALOG_MODAL,
-            gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, message)
+    dialog = Gtk.MessageDialog(None, Gtk.DialogFlags.MODAL,
+            Gtk.MessageType.ERROR, Gtk.ButtonsType.OK, message)
     dialog.run()
     dialog.destroy()
 
@@ -87,7 +88,7 @@ def has_ancestor(widget, wtype):
 def manual_lookup():
     '''Choose the manual to open based on LANGUAGE'''
     available_languages = ['en']
-    base_url = 'http://terminator-gtk2.readthedocs.io/%s/latest/'
+    base_url = 'http://terminator-gtk3.readthedocs.io/%s/latest/'
     target = 'en'   # default to English
     if 'LANGUAGE' in os.environ:
         languages = os.environ['LANGUAGE'].split(':')
@@ -149,14 +150,9 @@ def shell_lookup():
 
 def widget_pixbuf(widget, maxsize=None):
     """Generate a pixbuf of a widget"""
-    if gtk.gtk_version < (2, 14):
-        return(None)
-
-    pixmap = widget.get_snapshot()
-    (width, height) = pixmap.get_size()
-    pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, False, 8, width, height)
-    pixbuf.get_from_drawable(pixmap, pixmap.get_colormap(), 0, 0, 0, 0, width,
-            height)
+    # FIXME: Can this be changed from using "import cairo" to "from gi.repository import cairo"?
+    window = widget.get_window()
+    width, height = window.get_width(), window.get_height()
 
     longest = max(width, height)
 
@@ -166,8 +162,18 @@ def widget_pixbuf(widget, maxsize=None):
     if not maxsize or (width * factor) > width or (height * factor) > height:
         factor = 1
 
-    scaledpixbuf = pixbuf.scale_simple(int(width * factor), int(height * factor), gtk.gdk.INTERP_BILINEAR)
+    preview_width, preview_height = int(width * factor), int(height * factor)
 
+    preview_surface = Gdk.Window.create_similar_surface(window,
+        cairo.CONTENT_COLOR, preview_width, preview_height)
+
+    cairo_context = cairo.Context(preview_surface)
+    cairo_context.scale(factor, factor)
+    Gdk.cairo_set_source_window(cairo_context, window, 0, 0)
+    cairo_context.paint()
+
+    scaledpixbuf = Gdk.pixbuf_get_from_surface(preview_surface, 0, 0, preview_width, preview_height);
+    
     return(scaledpixbuf)
 
 def get_config_dir():
@@ -325,3 +331,10 @@ def spawn_new_terminator(cwd, args):
       
     dbg("Spawning: %s" % cmd)
     subprocess.Popen([cmd]+args)
+
+def display_manager():
+    """Try to detect which display manager we run under"""
+    if os.environ.get('WAYLAND_DISPLAY'):
+        return 'WAYLAND'
+    # Fallback assumption of X11
+    return 'X11'
