@@ -6,12 +6,11 @@
 import copy
 import time
 import uuid
-import pygtk
-pygtk.require('2.0')
-import gobject
-import gtk
+import gi
+from gi.repository import GObject
+from gi.repository import Gtk, Gdk
 
-from util import dbg, err, make_uuid
+from util import dbg, err, make_uuid, display_manager
 import util
 from translation import _
 from version import APP_NAME
@@ -19,14 +18,17 @@ from container import Container
 from factory import Factory
 from terminator import Terminator
 
-try:
-    import keybinder
-except ImportError:
-    err('Warning: python-keybinder is not installed. This means the \
+if display_manager() == 'X11':
+    try:
+        gi.require_version('Keybinder', '3.0')
+        from gi.repository import Keybinder
+        Keybinder.init()
+    except ImportError:
+        err('Warning: python-keybinder is not installed. This means the \
 hide_window shortcut will be unavailable')
 
 # pylint: disable-msg=R0904
-class Window(Container, gtk.Window):
+class Window(Container, Gtk.Window):
     """Class implementing a top-level Terminator window"""
 
     terminator = None
@@ -45,11 +47,11 @@ class Window(Container, gtk.Window):
 
     term_zoomed = False
     __gproperties__ = {
-            'term_zoomed': (gobject.TYPE_BOOLEAN,
+            'term_zoomed': (GObject.TYPE_BOOLEAN,
                             'terminal zoomed',
                             'whether the terminal is zoomed',
                             False,
-                            gobject.PARAM_READWRITE)
+                            GObject.PARAM_READWRITE)
     }
 
     def __init__(self):
@@ -58,11 +60,11 @@ class Window(Container, gtk.Window):
         self.terminator.register_window(self)
 
         Container.__init__(self)
-        gtk.Window.__init__(self)
-        gobject.type_register(Window)
+        GObject.GObject.__init__(self)
+        GObject.type_register(Window)
         self.register_signals(Window)
 
-        self.set_property('allow-shrink', True)
+#        self.set_property('allow-shrink', True)  # FIXME FOR GTK3, or do we need this actually?
         icon_to_apply=''
 
         self.register_callbacks()
@@ -79,8 +81,8 @@ class Window(Container, gtk.Window):
             if options.role:
                 self.set_role(options.role)
             
-            if options.classname is not None:
-                self.set_wmclass(options.classname, self.wmclass_class)
+#            if options.classname is not None:
+#                self.set_wmclass(options.classname, self.wmclass_class)
             
             if options.forcedicon is not None:
                 icon_to_apply = options.forcedicon
@@ -120,18 +122,19 @@ class Window(Container, gtk.Window):
         # Attempt to grab a global hotkey for hiding the window.
         # If we fail, we'll never hide the window, iconifying instead.
         if self.config['keybindings']['hide_window'] != None:
-            try:
-                self.hidebound = keybinder.bind(
-                    self.config['keybindings']['hide_window'],
-                    self.on_hide_window)
-            except (KeyError, NameError):
-                pass
+            if display_manager() == 'X11':
+                try:
+                    self.hidebound = Keybinder.bind(
+                        self.config['keybindings']['hide_window'],
+                        self.on_hide_window)
+                except (KeyError, NameError):
+                    pass
 
-            if not self.hidebound:
-                err('Unable to bind hide_window key, another instance/window has it.')
-                self.hidefunc = self.iconify
-            else:
-                self.hidefunc = self.hide
+                if not self.hidebound:
+                    err('Unable to bind hide_window key, another instance/window has it.')
+                    self.hidefunc = self.iconify
+                else:
+                    self.hidefunc = self.hide
 
     def apply_config(self):
         """Apply various configuration options"""
@@ -168,14 +171,14 @@ class Window(Container, gtk.Window):
 
     def apply_icon(self, requested_icon):
         """Set the window icon"""
-        icon_theme = gtk.icon_theme_get_default()
-        icon_name_list = [self.wmclass_name, APP_NAME]
+        icon_theme = Gtk.IconTheme.get_default()
+        icon_name_list = [APP_NAME]   # disable self.wmclass_name, n/a in GTK3
 
         if requested_icon:
             try:
                 self.set_icon_from_file(requested_icon)
                 return
-            except (NameError, gobject.GError):
+            except (NameError, GObject.GError):
                 dbg('Unable to load %s icon as file' % (repr(requested_icon)))
 
             icon_name_list.insert(0, requested_icon)
@@ -188,7 +191,7 @@ class Window(Container, gtk.Window):
             else:
                 dbg('Unable to load %s icon' % (icon_name))
 
-        icon = self.render_icon(gtk.STOCK_DIALOG_INFO, gtk.ICON_SIZE_BUTTON)
+        icon = self.render_icon(Gtk.STOCK_DIALOG_INFO, Gtk.IconSize.BUTTON)
         self.set_icon(icon)
 
     def on_key_press(self, window, event):
@@ -205,9 +208,9 @@ class Window(Container, gtk.Window):
                 self.set_fullscreen(not self.isfullscreen)
             elif mapping == 'close_window':
                 if not self.on_delete_event(window,
-                        gtk.gdk.Event(gtk.gdk.DELETE)):
+                        Gdk.Event.new(Gdk.EventType.DELETE)):
                     self.on_destroy_event(window,
-                            gtk.gdk.Event(gtk.gdk.DESTROY))
+                            Gdk.Event.new(Gdk.EventType.DESTROY))
             else:
                 return(False)
             return(True)
@@ -276,7 +279,7 @@ class Window(Container, gtk.Window):
         """Display a confirmation dialog when the user is closing multiple
         terminals in one window"""
         
-        return(not (self.construct_confirm_close(window, type) == gtk.RESPONSE_ACCEPT))
+        return(not (self.construct_confirm_close(window, type) == Gtk.ResponseType.ACCEPT))
 
     def on_destroy_event(self, widget, data=None):
         """Handle window destruction"""
@@ -302,7 +305,7 @@ class Window(Container, gtk.Window):
             self.show()
             self.grab_focus()
             try:
-                t = gtk.gdk.x11_get_server_time(self.window)
+                t = GdkX11.x11_get_server_time(self.window)
             except AttributeError:
                 t = 0
             self.window.focus(t)
@@ -314,9 +317,9 @@ class Window(Container, gtk.Window):
     def on_window_state_changed(self, window, event):
         """Handle the state of the window changing"""
         self.isfullscreen = bool(event.new_window_state & 
-                                 gtk.gdk.WINDOW_STATE_FULLSCREEN)
+                                 Gdk.WindowState.FULLSCREEN)
         self.ismaximised = bool(event.new_window_state &
-                                 gtk.gdk.WINDOW_STATE_MAXIMIZED)
+                                 Gdk.WindowState.MAXIMIZED)
         dbg('Window::on_window_state_changed: fullscreen=%s, maximised=%s' \
                 % (self.isfullscreen, self.ismaximised))
 
@@ -368,21 +371,17 @@ class Window(Container, gtk.Window):
 
         screen = self.get_screen()
         if value:
-            dbg('setting rgba colormap')
-            colormap = screen.get_rgba_colormap()
-        else:
-            dbg('setting rgb colormap')
-            colormap = screen.get_rgb_colormap()
-
-        if colormap:
-            self.set_colormap(colormap)
+            dbg('setting rgba visual')
+            visual = screen.get_rgba_visual()
+            if visual:
+                self.set_visual(visual)
     
     def show(self, startup=False):
         """Undo the startup show request if started in hidden mode"""
         #Present is necessary to grab focus when window is hidden from taskbar.
         #It is important to call present() before show(), otherwise the window
         #won't be brought to front if an another application has the focus.
-        #Last note: present() will implicitly call gtk.Window.show()
+        #Last note: present() will implicitly call Gtk.Window.show()
         self.present()
 
         #Window must be shown, then hidden for the hotkeys to be registered
@@ -392,9 +391,9 @@ class Window(Container, gtk.Window):
 
 
     def add(self, widget, metadata=None):
-        """Add a widget to the window by way of gtk.Window.add()"""
+        """Add a widget to the window by way of Gtk.Window.add()"""
         maker = Factory()
-        gtk.Window.add(self, widget)
+        Gtk.Window.add(self, widget)
         if maker.isinstance(widget, 'Terminal'):
             signals = {'close-term': self.closeterm,
                        'title-change': self.title.set_title,
@@ -423,8 +422,8 @@ class Window(Container, gtk.Window):
             widget.grab_focus()
 
     def remove(self, widget):
-        """Remove our child widget by way of gtk.Window.remove()"""
-        gtk.Window.remove(self, widget)
+        """Remove our child widget by way of Gtk.Window.remove()"""
+        Gtk.Window.remove(self, widget)
         self.disconnect_child(widget)
         return(True)
 
@@ -480,10 +479,10 @@ class Window(Container, gtk.Window):
         for term in order:
             container.add(term)
         container.show_all()
-        sibling.grab_focus()
         
-        while gtk.events_pending():
-            gtk.main_iteration_do(False)
+        while Gtk.events_pending():
+            Gtk.main_iteration_do(False)
+        sibling.grab_focus()
         self.set_pos_by_ratio = False
 
 
@@ -534,8 +533,6 @@ class Window(Container, gtk.Window):
         """Rotate children in this window"""
         self.set_pos_by_ratio = True
         maker = Factory()
-        # collect all paned children in breadth-first order
-        paned = []
         child = self.get_child()
 
         # If our child is a Notebook, reset to work from its visible child
@@ -544,19 +541,18 @@ class Window(Container, gtk.Window):
             child = child.get_nth_page(pagenum)
 
         if maker.isinstance(child, 'Paned'):
-            paned.append(child)
-        for p in paned:
-            for child in p.get_children():
-                if child not in paned and maker.isinstance(child, 'Paned'):
-                    paned.append(child)
-        # then propagate the rotation
-        for p in paned:
-            p.rotate(widget, clockwise)
-        self.show_all()
-        widget.grab_focus()
-        
-        while gtk.events_pending():
-            gtk.main_iteration_do(False)
+            parent = child.get_parent()
+            # Need to get the allocation before we remove the child,
+            # otherwise _sometimes_ we get incorrect values.
+            alloc = child.get_allocation()
+            parent.remove(child)
+            child.rotate_recursive(parent, alloc.width, alloc.height, clockwise)
+
+            self.show_all()
+            while Gtk.events_pending():
+                Gtk.main_iteration_do(False)
+            widget.grab_focus()
+
         self.set_pos_by_ratio = False
 
     def get_visible_terminals(self):
@@ -599,7 +595,7 @@ class Window(Container, gtk.Window):
         if self.pending_set_rough_geometry_hint == True:
             return
         self.pending_set_rough_geometry_hint = True
-        gobject.idle_add(self.do_deferred_set_rough_geometry_hints)
+        GObject.idle_add(self.do_deferred_set_rough_geometry_hints)
 
     def do_deferred_set_rough_geometry_hints(self):
         self.pending_set_rough_geometry_hint = False
@@ -650,8 +646,12 @@ class Window(Container, gtk.Window):
         dbg('setting geometry hints: (ewidth:%s)(eheight:%s),\
 (fwidth:%s)(fheight:%s)' % (extra_width, extra_height, 
                             font_width, font_height))
-        self.set_geometry_hints(self, -1, -1, -1, -1, extra_width,
-                extra_height, font_width, font_height, -1.0, -1.0)
+        geometry = Gdk.Geometry()
+        geometry.base_width = extra_width
+        geometry.base_height = extra_height
+        geometry.width_inc = font_width
+        geometry.height_inc = font_height
+        self.set_geometry_hints(self, geometry, Gdk.WindowHints.BASE_SIZE | Gdk.WindowHints.RESIZE_INC)
 
     def tab_change(self, widget, num=None):
         """Change to a specific tab"""
@@ -833,7 +833,7 @@ class Window(Container, gtk.Window):
 
             if len(winners) > 1:
                 # Break an n-way tie using the cursor position
-                term_alloc = terminal.allocation
+                term_alloc = terminal.get_allocation()
                 cursor_x = term_alloc.x + term_alloc.width / 2
                 cursor_y = term_alloc.y + term_alloc.height / 2
 
